/* ------------------------------------------------------------------------*/
/* COPYRIGHT (c) 1992, 1995 CONTEMPORARY CONTROL SYSTEMS, INC.             */
/* ------------------------------------------------------------------------*/
/* Project:       20020 DRIVER                                             */
/* Filename:      SNUP20.c                                                 */
/* Description:   snooper program for the 20020                            */
/* Version:       2.00                                                     */
/* Author:        Mike Justice                                             */
/* History:       ORIGINAL 09/14/92                                        */
/* ------------------------------------------------------------------------*/
#include "arcdef.h"
#include "msc.h"

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>

/* ------------------------------------------------------------------------- */
/* external variable definitions                                             */
/* ------------------------------------------------------------------------- */
extern USIGN16 arc20_port;
extern USIGN16 arc20_swport;
extern USIGN8  arc20_intr;
extern USIGN8  arc20_level;

extern  USIGN8  arc20_stat;
extern  USIGN8  arc20_config;
extern  USIGN8  arc20_diag;
extern  USIGN8  arc20_cmd;

extern  USIGN16 arc20_interrupt_reg;
extern  USIGN16 arc20_status_reg;
extern  USIGN16 arc20_command_reg;
extern  USIGN16 arc20_diag_reg;
extern  USIGN16 arc20_addr_hi_reg;
extern  USIGN16 arc20_addr_low_reg;
extern  USIGN16 arc20_packet_mem;
extern  USIGN16 arc20_config_reg;
extern  USIGN16 arc20_tentid_reg;
extern  USIGN16 arc20_nextid_reg;
extern  USIGN16 arc20_nodeid_reg;
extern  USIGN16 arc20_setup_reg;

/* ------------------------------------------------------------------------- */
/* global variable definition                                                */
/* ------------------------------------------------------------------------- */
USIGN8  arc20_nextid;
USIGN8  arc20_nodeid;
USIGN8  arc20_tentid;
USIGN8 map1[256],map2[256];

/* ------------------------------------------------------------------------- */
/* function prototypes */
/* ------------------------------------------------------------------------- */
#include "arc20.h"
void net_map(USIGN8 *data_ptr);
void net_tokens(USIGN8 ntokens);


/****************************************************************************
 * FUNCTION NAME: main
 *
 * DESCRIPTION  : simple read routine
 *
 * RETURN VALUE : status
 ***************************************************************************/
void main(int argc, char *argv[])
{
    USIGN8 match,found;

    USIGN8 status,node_id;
    char *s,cc;
    int i;

    /* signon message */
    printf("Contemporary Control Systems, Inc.\n");
    printf("ARC20 Snooper Test Routine V2.20, Copyright (c) 1993-1996\n");

    /* check arguments */
    if (argc > 1) {
        while (--argc > 0 && (*++argv)[0] == '-') {
            for (s = argv[0] + 1; *s != '\0'; s++) {
                switch (*s) {
                /* port address */
                case 'p':
                case 'P':
                    i = 1;
                    arc20_port = 0;
                    cc = *(s+1);
                    while (cc != '\0') {
                        if ((cc >= '0') && (cc <= '9')) {
                            arc20_port = arc20_port * 16 + (cc - 0x30);
                        }
                        else if ((cc >= 'a') && (cc <= 'f')) {
                            arc20_port = arc20_port * 16 + (cc - 0x57);
                        }
                        else if ((cc >= 'A') && (cc <= 'F')) {
                            arc20_port = arc20_port * 16 + (cc - 0x37);
                        }
                        i++;
                        cc = *(s+i);
                    }
                    break;
                
                /* interrupt enable */
                case 'i':
                case 'I':
                    arc20_intr = FALSE;
                    if (*(s+1) == '1') arc20_intr = TRUE;
                    break;
                /* interrupt level */
                case 'l':
                case 'L':
                    arc20_level = 0;
                    if ((*(s+1) >= '0') && (*(s+1) <= '9'))
                        arc20_level = *(s+1) - (USIGN8)0x30;
                    break;
                /* switch address */
                case 's':
                case 'S':
                    i = 1;
                    arc20_swport = 0;
                    cc = *(s+1);
                    while (cc != '\0') {
                        if ((cc >= '0') && (cc <= '9')) {
                            arc20_swport = arc20_swport * 16 + (cc - 0x30);
                        }
                        else if ((cc >= 'a') && (cc <= 'f')) {
                            arc20_swport = arc20_swport * 16 + (cc - 0x57);
                        }
                        else if ((cc >= 'A') && (cc <= 'F')) {
                            arc20_swport = arc20_swport * 16 + (cc - 0x37);
                        }
                        i++;
                        cc = *(s+i);
                    }
                    break;
                
                case 'h':
                case 'H':
                case '?':
                    printf("Options: -p<port address> -s<switch address> -i<interrupt> -l<level>\n");
                    printf("         port address default = 2e0\n");
                    printf("         switch address default = 2e9\n");
                    printf("         interrupt 0=off,1=on\n");
                    printf("         level = 0-7\n");
                    exit(0);
                    break;
                default:
                    break;
                }
            }
        }
    }

    /* initialize the driver */
    status = arc20_init();
    if (status == 0) {
        printf("Initialization Failure\n");
        exit(0);
    }
    else {
        printf("Node ID = %d (%XH) Initialized\n\n",status,status);
        arc20_nodeid = status;
        printf("Network Nodes = ");
        node_id = status;
    }

    found = FALSE;
    while (found == FALSE)
    {
        net_map( map1 );
        net_map( map2 );

        match = TRUE;
        for (i=0; i<256; i++)
        {
            if (map1[i] != map2[i]) match = FALSE;
        }    

        if (match == TRUE) 
        {
            found = TRUE;
            for (i=0; i<256; i++)
            {
            	if (map1[i] > 0) printf("%3d(%2X) ",map1[i],map1[i]);
            }
        }
    }

    /* exit to dos */
    arc20_exit();
    printf("\nExiting to DOS\n");
    exit(0);
}

/****************************************************************************
 * FUNCTION NAME: net_map
 *
 * DESCRIPTION  : read the network map
 *
 * RETURN VALUE : status
 ***************************************************************************/
void net_map(USIGN8 *data_ptr)
{
    USIGN8 waittokens;
	USIGN16 nodecount;

    /* read the next ID */
    arc20_config = (USIGN8)INP(arc20_config_reg);
    arc20_config = (USIGN8)((arc20_config & SUB_AD) | NEXT_ID_REG);
    OUTP(arc20_config_reg, arc20_config);
    arc20_nextid = (USIGN8)INP(arc20_nextid_reg);    
    if (arc20_nextid == 0) arc20_nextid = arc20_nodeid;

    /* read the current tentative ID */
    arc20_config = (USIGN8)INP(arc20_config_reg);
    arc20_config = (USIGN8)((arc20_config & SUB_AD) | TENT_ID_REG);
    OUTP(arc20_config_reg, arc20_config);
    arc20_tentid = (USIGN8)INP(arc20_tentid_reg);    

    nodecount = 0;
    while (nodecount <= 255) 
    {
        /* output new address */
        OUTP(arc20_tentid_reg,(USIGN8)nodecount);

        waittokens = 255;
        while (waittokens > 0) 
        {
            /* wait 1 token rotation */
            net_tokens(1);

            /* look for id - if set then set bit in node map */
            if (((arc20_diag & TENT_ID) == TENT_ID) || 
                ((USIGN8)nodecount == arc20_nextid) ||
                ((USIGN8)nodecount == arc20_nodeid)) 
            {
                *(data_ptr+nodecount) = (USIGN8)nodecount;

                 /* time to exit loop */
                 waittokens = 1;
            }
            else *(data_ptr+nodecount) = 0;

            /* token counter */
            waittokens--;
        }

        /* get node */
        nodecount++;
    }
}


/****************************************************************************
 * FUNCTION NAME: net_tokens
 *
 * DESCRIPTION  : wait x token rotations
 *
 * RETURN VALUE : none
 ***************************************************************************/
void net_tokens(USIGN8 ntokens)
{
    USIGN8 recon_cnt = 0;

    /* clear dupid seen bit */
    arc20_diag = (USIGN8)INP(arc20_diag_reg);
    arc20_diag = (USIGN8)INP(arc20_diag_reg);

    /* clear RECON flag */
    OUTP(arc20_command_reg, 0x1e); 

    /* wait x rotations */
    while (ntokens > 0) 
    {
        /* wait for a token rotation */
        while (((arc20_diag & DUPID) == 0) && (recon_cnt <= 3) && 
               ((arc20_diag & MYRECON) != MYRECON))  
        {
            arc20_stat = (USIGN8)INP(arc20_status_reg);
            if ((arc20_stat & RECON) == RECON) 
            {
               recon_cnt++;
               OUTP(arc20_command_reg, 0x1e);
            }
            arc20_diag = (USIGN8)INP(arc20_diag_reg);
        }

        /* clear the DUPID bit */
        arc20_diag = (USIGN8)INP(arc20_diag_reg);
        recon_cnt = 0;

        /* decr the number of tokens */
        ntokens--;
    }
}
